"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const yeoman_generator_1 = require("yeoman-generator");
const questions_1 = require("./utils/questions");
const { error, loading, info, } = require("prettycli");
const fs = require("fs");
const endMessage_1 = require("./utils/endMessage");
const pkgjsonUtils_1 = require("./utils/pkgjsonUtils");
const CFonts = require("cfonts");
const projectFeatures_1 = require("./utils/projectFeatures");
class StarterGenerator extends yeoman_generator_1.default {
    constructor(args, opts) {
        super(args, opts);
        CFonts.say("Create|Web App", {
            font: "simple3d",
            align: "left",
            colors: ["yellowBright"],
            background: "transparent",
            letterSpacing: 1,
            lineHeight: 1,
            space: true,
            maxLength: "0",
        });
    }
    prompting() {
        return __awaiter(this, void 0, void 0, function* () {
            const done = this.async();
            // Have Yeoman greet the user.
            return this.prompt(questions_1.default)
                .then((answer) => {
                this.answers = answer;
                const tmppkgjsonFromAnswer = {
                    name: answer.projectName,
                    author: answer.author,
                    license: answer.license,
                    description: answer.description,
                    repo: answer.githubRepo,
                    keywords: [answer.stack],
                };
                loading("CREATING", "Cooking your project ! Please wait ...");
                fs.readFile(path.join(__dirname, `../generators/app/templates/${answer.stack}/package.json`), {
                    encoding: "utf-8",
                }, (err, data) => __awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        error("File Not Found, Internal Error.Please Report an issue");
                        return;
                    }
                    else {
                        let pkgjsonFromTemplate = JSON.parse(data);
                        const { bundler, } = answer;
                        pkgjsonFromTemplate = bundler == "Webpack" ? pkgjsonUtils_1.createPkgJsonWithWebpack(pkgjsonFromTemplate, tmppkgjsonFromAnswer) : bundler == "Parcel" ? pkgjsonUtils_1.createPkgJsonWithParcel(pkgjsonFromTemplate, tmppkgjsonFromAnswer) : pkgjsonFromTemplate;
                        // Now will use this this.pkjson to write to the package.json file after copying of the files
                        this.pkgjson = Object.assign({}, pkgjsonFromTemplate);
                        done();
                    }
                }));
            });
        });
    }
    writing() {
        info("DONE", "Copying your project.....");
        this.fs.copy(this.templatePath(path.join(__dirname, `../generators/app/templates/${this.answers.stack}`)), this.destinationPath(path.resolve(this.answers.projectName)));
        info("DONE", "Writing your dependencies and scripts.....");
        this.fs.writeJSON(`${path.resolve(__dirname, this.answers.projectName)}/package.json`, this.pkgjson);
    }
    install() {
        // this.installDependencies({
        //   npm: true,
        //   bower: false,
        // });
    }
    end() {
        CFonts.say("Thank|You", {
            font: "block",
            align: "left",
            colors: ["yellowBright"],
            background: "transparent",
            letterSpacing: 1,
            lineHeight: 1,
            space: true,
            maxLength: "0",
        });
        endMessage_1.endMessage(this.answers.projectName, projectFeatures_1.features[this.answers.stack]);
    }
}
exports.default = StarterGenerator;
